@@ -2,4 +2,6 @@ mod test_access_management;
 mod test_adding_files;
 mod test_adding_tags;
 mod test_adding_urls;
+mod test_managing_cookies_and_http_headers;
+mod test_managing_pages;
 mod test_searching_and_fetching_files;
